/* Linklist.c */
#include "linklist.h"

#ifdef LINUX
#define ErrorMessage printf
#endif

#ifdef TESTING

#ifdef LINUX
static int parseFile(char *);
#endif

/* Global variables */
linkList *llScsi;
linkList *llEth;
linkList *llCDRom;

void main(void)
{
	char *fileName = "image.dat";
	llScsi = (linkList *)malloc(sizeof(linkList));
	llEth = (linkList *)malloc(sizeof(linkList));
	llCDRom = (linkList *)malloc(sizeof(linkList));

   createList(llScsi);
	createList(llEth);
	createList(llCDRom);

	if (!parseFile(fileName))
	{
		printf("Couldn't parse file");
		destroyList(llScsi);
		destroyList(llEth);
		destroyList(llCDRom);
		return;
	}
}

#endif

void createList( linkList *ll )
{
	ll->nFirst = ll->nLast = ll->nCurrent = LPNULL;
}

void destroyList( linkList *ll )
{
	node *n;
	node *tmp;
	
	n = ll->nFirst;
	while ( n )
	{
		free(n->pMem);
		tmp = n->nNext;
		free(n);
		n = tmp;
	}
}

#ifndef LINUX
void __far *next(linkList *ll)
#else
void *next(linkList *ll)
#endif
{
	if (empty(ll) || (ll->nCurrent == LPNULL))
		return NULL;

	ll->nCurrent = ll->nCurrent->nNext;
	
	if (ll->nCurrent)
		return ll->nCurrent->pMem;
	else
		return NULL;
}

#ifndef LINUX
void __far *prev(linkList *ll)
#else
void *prev(linkList *ll)
#endif
{
	if (empty(ll))
		return NULL;
		
	if (!ll->nCurrent->nPrev)
		return NULL;
		
	ll->nCurrent = ll->nCurrent->nPrev;
	
	return ll->nCurrent->pMem;
}

int ll_remove(linkList *ll)
{
	node *nTemp;

	if (empty(ll))
		return FALSE;
		
	if (!ll->nCurrent)
		return FALSE;
                                 
	/* 1 node in list */
	if (ll->nFirst == ll->nLast)
	{
		free(ll->nCurrent->pMem);
		free (ll->nFirst);
		ll->nFirst = ll->nLast = ll->nCurrent = LPNULL;				/* null the list out*/
	}
	/* deleting the first element with >1 element */
	else if (ll->nCurrent == ll->nFirst)
	{
		free(ll->nCurrent->pMem);
		ll->nCurrent = ll->nFirst->nNext;					/* move to the next node */
		ll->nCurrent->nPrev = LPNULL;					/* unattach it */
		free (ll->nFirst);

		ll->nFirst = ll->nCurrent;						/* set beginning of list */
	}
	/* deleting the last node with >1 element */
	else if (ll->nCurrent == ll->nLast)
	{
		free(ll->nCurrent->pMem);
		ll->nCurrent = ll->nLast->nPrev;					/* move to the prev node */
		ll->nCurrent->nNext = LPNULL;					/* unattach it */
		free (ll->nLast);
		ll->nLast = ll->nCurrent;							/* set end of list */
	}
	/* we're killing a middle node*/
	else
	{
		ll->nCurrent->nPrev->nNext = ll->nCurrent->nNext;		/* close up one side */
		ll->nCurrent->nNext->nPrev = ll->nCurrent->nPrev;		/* and the other */
		free(ll->nCurrent->pMem);
		nTemp = ll->nCurrent->nNext;						/* hold the next */
		free (ll->nCurrent);
		ll->nCurrent = nTemp;	    						/* and reset the current */
	}
	
	return (ll->nCurrent ? TRUE : FALSE);
}

#ifndef LINUX
void __far *first(linkList *ll)
#else
void *first(linkList *ll)
#endif
{
	if (!ll->nFirst)
		return NULL;
		
	ll->nCurrent = ll->nFirst;
	
	return ll->nCurrent->pMem;
}

#ifndef LINUX
void __far *last(linkList *ll)
#else
void *last(linkList *ll)
#endif
{
	if (!ll->nLast)
		return NULL;
		
	ll->nCurrent = ll->nLast;
	
	return ll->nCurrent->pMem;
}

#ifndef LINUX
int add(linkList *ll, void __far *pMem)
#else
int add(linkList *ll, void *pMem)
#endif
{
	node *nTemp;
	
	nTemp = (node *)malloc(sizeof(node));

	if (!nTemp)
		return FALSE;
	
	nTemp->pMem = pMem;
	nTemp->nPrev = LPNULL;
	nTemp->nNext = LPNULL;
	
	/* now add the sucker */
	if (empty(ll))
	{
		ll->nFirst = ll->nLast = ll->nCurrent = nTemp;
		return TRUE;
	}
	
	nTemp->nPrev = ll->nLast;
	ll->nLast->nNext = nTemp;
	ll->nLast = nTemp;
	ll->nCurrent = ll->nLast;    
	
	return TRUE;
}

int empty(linkList *ll)
{
	return (int)!ll->nFirst;
}

#ifdef LINUX
static int parseFile( char *fileName )
{
/*	char *fileName = "image.dat"; */
	FILE *fpIn;
	char *cDelim = ";";
	char *buf = (char *)malloc(sizeof(char) * MAXLINELEN);
	int iLen = 0;
	char *tmp;
	char *ermsg = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *scsiElem;
	char *ethElem;
	char *cdElem;

	if ( !buf )
	{
		ErrorMessage("parseFile: Malloc of buf failed!");
		return 0;
	}
	
	fpIn = fopen(fileName, "r");
	if ( !fpIn)
	{
      strcpy(ermsg, "Can not open\n");
      strcat(ermsg, fileName);
      ErrorMessage(ermsg);
		free(buf);

		return 0;
	}

	/* Now parse through the file.
	   Lines beginning with # are comments
	   Other lines are in the form XXXXX; SCSI; ETH; CDROM */
	for( ; ; )
	{
		/* Read a line out of the file */
		fgets( buf, MAXLINELEN, fpIn );
		
		/* Get actual length of the string it includes \n */
		iLen = strlen(buf);

		/* Check the first character.  If it's a # skip it*/
		if ( buf[0] != '#' )
		{
			/* begin parsing and generating the 3 lists. */
			char *cScsiDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cEthDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cCDDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cNumber = (char *)malloc(sizeof(char) * MAXFIELDLEN);

			/* check for malloc errors! */
			if (!cScsiDriver || !cEthDriver || !cCDDriver || !cNumber )
			{
		      ErrorMessage("Malloc of cScsiDriver, cEthDriver, cCDDriver or cNumber failed!");
				
				if (cScsiDriver) free(cScsiDriver);
				if (cEthDriver) free(cEthDriver);
				if (cCDDriver) free(cCDDriver);
				if (cNumber) free(cNumber);
				free(buf);
				
				return 0;
			}

			/* Parse the string breaking at delim */
			cNumber = strtok(buf,cDelim);
			cScsiDriver = strtok(NULL,cDelim);
			cEthDriver = strtok(NULL,cDelim);
			cCDDriver = strtok(NULL,cDelim);

			/* Make a list of strings */
			scsiElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(scsiElem, cScsiDriver);
			add(llScsi, scsiElem);

			ethElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(ethElem, cEthDriver);
			add(llEth, ethElem);

			cdElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(cdElem, cCDDriver);
			add(llCDRom, cdElem);
			
			// clean up
			free(cScsiDriver);
			free(cEthver);
			free(cCDiver);
			free(cNumber);
		}
		
		/* when we get to eof quit. */
		if (feof(fpIn)) break;
	}
	
	free(buf);
	fclose(fpIn);

#ifdef TESTING
	/* Ok to test, we'll print the scsi drivers */
	tmp = (char *)first(llScsi);
	while (tmp)
	{
      strcpy(ermsg, "Scsi Element is ");
      strcat(ermsg, tmp);
      ErrorMessage(ermsg);
/*      printf("Element is %s\n",tmp); */
      tmp = (char *)next(llScsi);
	}	

	tmp = (char *)first(llEth);
	while (tmp)
	{
      strcpy(ermsg, "Eth Element is ");
      strcat(ermsg, tmp);
      ErrorMessage(ermsg);
/*      printf("Element is %s\n",tmp); */
      tmp = (char *)next(llEth);
	}	

	tmp = (char *)first(llCDRom);
	while (tmp)
	{
      strcpy(ermsg, "CDRom Element is ");
      strcat(ermsg, tmp);
      ErrorMessage(ermsg);
/*      printf("Element is %s\n",tmp); */
      tmp = (char *)next(llCDRom);
	}	
#endif

	return 1;	
}
#endif
