#include "redhat.h"

/* Things to be aware of:
	main() - fileName must be file to parse image info out of
	RunRawrite() - cmdName is name and location of our modified rawrite executable
	RunRawrite() - imgname is a created path to the images from the cdDriveletter
						and the imgnumber but the name of the image must be modified
						by us to be the beginning part before the number
						AND there are 2 locations that need to be modified, 1 for boot
						1 for root disks
	
	You CAN NOT interrupt dflat with Ctrl-C otherwise it freaks the system out
	and you'll need to reboot.  I don't know how to disable ctrl-c capability
*/
 
// Global application name
char DFlatApplication[] = "Caldera Network Desktop Disk Utility";

// Our global link list for storing elements
linkList *llScsi;
linkList *llEth;
linkList *llCDRom;

// Global to hold Drive letters
static char cFloppyLetter[3] = { 0 };
static char cCDRomLetter[3] = { 0 };
static char cDosDrivePath[80] = { 0 };
static char *scsiInclude;
static char *ethInclude;
static char *cdromInclude;
static int scsiNum = -1;
static int ethNum = -1;
static int cdNum = -1;

// Needed otherwise the dflat.lib freaks
char **Argv;

// Main program.
void main(short argc, char *argv[])
{
	WINDOW wnd;
	static BOOL bFirst = TRUE;
		
	if (!init_messages()) return;
	
	Argv = argv;

	if (!LoadConfig())
		cfg.ScreenLines = SCREENHEIGHT;

    wnd = CreateWindow(APPLICATION,
                        "Caldera Network Desktop Disk Utility " VERSION,
                        0, 0, -1, -1,
                        &MainMenu,
                        NULL,
                        RHSProc,
                        MOVEABLE  |
                        SIZEABLE  |
                        HASBORDER |
								MINMAXBOX |
                        HASSTATUSBAR
                        );

	LoadHelpFile();
	SendMessage(wnd, SETFOCUS, TRUE, 0);

	scsiInclude = (char *)malloc(sizeof(char) * MAXFIELDLEN );
	ethInclude = (char *)malloc(sizeof(char) * MAXFIELDLEN );
	cdromInclude = (char *)malloc(sizeof(char) * MAXFIELDLEN );
   
   scsiInclude[0] = '\0';
   ethInclude[0] = '\0';
   cdromInclude[0] = '\0';
   
	// First popup introduction box
	if (bFirst)
	{
		bFirst = FALSE;
		SendMessage(wnd,COMMAND, ID_MAKEDISKS, 0);
	}

	while (dispatch_message())
		;
}

/* ------- window processing module for the
                    rhs application window ----- */
static short RHSProc(WINDOW wnd,MESSAGE msg,PARAM p1,PARAM p2)
{
	short rtn;

	switch (msg)
	{
		case CREATE_WINDOW:
			rtn = DefaultWndProc(wnd, msg, p1, p2);
			return rtn;
		break;
		case COMMAND:
			switch ((short)p1)    
			{
				case ID_MAKEDISKS:
					MakeDisks(wnd);
					return TRUE;
				case ID_FIPS:
					RunFips(wnd);
					return TRUE;
				case ID_DEFRAG:
					RunDefrag(wnd);
					return TRUE;
				case ID_FDISK:
					RunFdisk(wnd);
					return TRUE;
				case ID_EXIT:
				{	
					if (!YesNoBox("Exit Caldera Network Desktop Disk Utility?"))
						return FALSE;
					else
					{
						destroyList(llScsi);
						destroyList(llEth);
						destroyList(llCDRom);

						free(llScsi);
						free(llEth);
						free(llCDRom);
						
						free(scsiInclude);
						free(ethInclude);
						free(cdromInclude);
					}
				}
				break;
				case ID_SCSI:
					SelectScsi(wnd);
					return TRUE;
				case ID_ETH:
					SelectEthernet(wnd);
					return TRUE;
				case ID_CDROM:
					SelectCDRom(wnd);
					return TRUE;
				case ID_ABOUT:
					MessageBox(
						"About Caldera Network Desktop Utility",
						"The Caldera Network Desktop Utility utilizes\n"
						"the D-Flat system which implements\n"
						"the SAA/CUA interface in a public domain\n"
						"C language library originally\n"
						"published in Dr. Dobb's Journal\n"
						"    ------------------------ \n");
					return TRUE;
				default:
				break;
			}
		break;
		default:
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void MakeDisks(WINDOW wnd)
{
	int iReturn;
	char *fileName = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *versionName = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *buf = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *tmpversion = (char *)malloc(sizeof(char) * MAXLINELEN);
	WINDOW mwnd;
	FILE *fpIn;
	
	// First: Popup the info dlg box
	iReturn = DialogBox(wnd, &MakeDisksDlg, TRUE, MakeDisksDlgProc);

	if ( iReturn )
	{
		// Have them select floppy and cdrom drives
		iReturn = DialogBox( wnd, &SelectDrives, TRUE, SelectDrivesDlgProc);

		if (iReturn)
		{
			/* ============ Now get the file ready parse it ================ */
			llScsi = (linkList *)malloc(sizeof(linkList));
			llEth = (linkList *)malloc(sizeof(linkList));
			llCDRom = (linkList *)malloc(sizeof(linkList));
			
			if (!llScsi || !llEth || !llCDRom)
			{
				if (llScsi) free(llScsi);
				if (llEth) free(llEth);
				if (llCDRom) free(llCDRom);
				
				ErrorMessage("Can not allocate memory for lists");
				free(fileName);
				free(versionName);
				free(buf);
				free(tmpversion);
				return;
			}

			createList(llScsi);
			createList(llEth);
			createList(llCDRom);
			
			mwnd = MomentaryMessage("Parsing Image file...\n     Please wait...");
			
			// Read the version out of version file
			sprintf(tmpversion, "%c:\\images\\version", cCDRomLetter[0]);
			fpIn = fopen(tmpversion, "r");
			fgets( buf, MAXLINELEN, fpIn );
			fclose(fpIn);
			buf[4] = '\0';
			
			sprintf(fileName, "%c:\\images\\%s\\image.idx",cCDRomLetter[0], buf);
			if (!parseFile(fileName))
			{
				ErrorMessage("Couldn't parse file");
				destroyList(llScsi);
				destroyList(llEth);
				destroyList(llCDRom);
				free(llScsi);
				free(llEth);
				free(llCDRom);
				free(fileName);
				free(tmpversion);
				free(buf);
				
				return;
			}

			SendMessage(mwnd, CLOSE_WINDOW, 0, 0);
			free(buf);
			
			
			// Ask them if they have a scsi drive
			iReturn = YesNoBox("Do you have a SCSI card?");
			
			if (iReturn)
			{
				// Display select scsi dlg box
				iReturn = DialogBox(wnd, &SelectScsiDlg, TRUE, SelectScsiDlgProc);
			}
			else
			{
				// Make scsi drive read like None was selected
				strcpy(scsiInclude," None\0");

				scsiNum = 0;
			}
			
			iReturn = YesNoBox("Do you have an ethernet card?");
			if (iReturn)
			{
				// display select ethernet dlg box
				iReturn = DialogBox(wnd, &SelectEthernetDlg, TRUE, SelectEthernetDlgProc);
			}
			else
			{
				// make eth driver read like none was selected
				strcpy(ethInclude," None\0");

				ethNum = 0;
			}
			
			// Do the cdrom thing here in a few.
			iReturn = DialogBox(wnd, &SelectCDRomDlg, TRUE, SelectCDRomDlgProc);
		}
	}

	RunRawrite(fileName);

	iReturn = DialogBox(wnd, &CalderaInfo, TRUE, CalderaInfoDlgProc);
	free(fileName);
	free(versionName);
	free(tmpversion);
	return;
}

static short MakeDisksDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	short rtn;
	
	switch (msg)
	{
		case CREATE_WINDOW:
		{
			rtn = DefaultWndProc(wnd, msg, p1, p2);
			return rtn;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_OK:
				{
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static short CalderaInfoDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	short rtn;
	
	switch (msg)
	{
		case CREATE_WINDOW:
		{
			rtn = DefaultWndProc(wnd, msg, p1, p2);
			return rtn;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_OK:
				{
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static short SelectDrivesDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	short rtn;
	
	switch (msg)
	{
		case CREATE_WINDOW:
		{
			// If drive letters have already been saved, display them in the
			// dlg box
			rtn = DefaultWndProc(wnd, msg, p1, p2);

			if ( cFloppyLetter || cFloppyLetter[0] )
			{
				PutItemText(wnd, ID_FLOPPYLETTER, cFloppyLetter);
			}

			if ( cCDRomLetter || cCDRomLetter[0] )
			{
				PutItemText(wnd, ID_CDROMLETTER, cCDRomLetter);
			}

			return rtn;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_OK:
				{
					char status[55];
					// Need to save drive letters
					GetItemText(wnd, ID_FLOPPYLETTER, cFloppyLetter, 2);
					GetItemText(wnd, ID_CDROMLETTER, cCDRomLetter, 2);
					
					// If not both drives pop up dlg box saying enter both
					if ( (!cFloppyLetter || !cFloppyLetter[0]) ||
							(!cCDRomLetter || !cCDRomLetter[0]) )
					{
						sprintf(status, "You MUST enter both Floppy and CD-Rom Drive Letters!");
						SendMessage(GetParent(wnd), ADDSTATUS, (PARAM) status, 0);

						return FALSE;
					}
					else
					{
						// If both, close dlg box
						// Let the app close the box for use below
					}
				}
				break;
				case ID_CANCEL:
				{
					// Do nothing, if we had drive letters keep them or keep noghing.
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void RunFips(WINDOW wnd)
{
	MessageBox("Caldera Network Desktop Utility", "Please read \\DOS\\FIPS.DOC on the CD for information on how to run FIPS.");

	return;
}

static void RunDefrag(WINDOW wnd)
{
	int iReturn;

	iReturn = DialogBox( wnd, &Warning, TRUE, WarningDlgProc);

	if ( iReturn )
	{	
		char *cmdName = (char *)calloc(sizeof(char), 80 );
	
		if (!cmdName ) 
		{
			ErrorMessage("Malloc failed");
			return;
		}
		
		if (!cDosDrivePath || !cDosDrivePath[0] )
			iReturn = DialogBox(wnd, &DosInfo, TRUE, DosInfoDlgProc);
		
		if ( iReturn )
		{
			strcat(cmdName, cDosDrivePath);
			strcat(cmdName,"\\defrag.exe");
		}
		
		if (!_spawnl(_P_WAIT, cmdName, cmdName, NULL))
		{
		}
		else
		{
			// Terminated abnormally
			ErrorMessage("defrag.exe couldn't be started.  Exit to DOS and try running it from there.");
		}
	
		free(cmdName);	
	
		SendMessage(wnd, PAINT, TRUE, 0);
		SendMessage(wnd, BORDER, TRUE, 0);
		SendMessage(wnd, TITLE, TRUE, 0);
	   SendMessage(wnd, SETFOCUS, TRUE, 0);
	}
	
	return;
}

static short WarningDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	short rtn;
	
	switch (msg)
	{
		case CREATE_WINDOW:
		{
			rtn = DefaultWndProc(wnd, msg, p1, p2);
			return rtn;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_OK:
				{
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void RunFdisk(WINDOW wnd)
{
	int iReturn;
	
	iReturn = DialogBox( wnd, &Warning, TRUE, WarningDlgProc);

	if ( iReturn )
	{	
		char *cmdName = (char *)calloc(sizeof(char), 80 );
		
		if (!cmdName ) 
		{
			ErrorMessage("Malloc failed");
			return;
		}
	
		if (!cDosDrivePath || !cDosDrivePath[0] )
			iReturn = DialogBox(wnd, &DosInfo, TRUE, DosInfoDlgProc);
		
		if ( iReturn )
		{
			strcat(cmdName, cDosDrivePath);
			strcat(cmdName,"\\fdisk.exe");
		}
		
		if (!_spawnl(_P_WAIT, cmdName, cmdName, NULL))
		{
		}
		else
		{
			// Terminated abnormally
			ErrorMessage("fdisk.exe couldn't be started.  Exit to DOS and try running it from there.");
		}
	
		free(cmdName);	
	
		SendMessage(wnd, PAINT, TRUE, 0);
		SendMessage(wnd, BORDER, TRUE, 0);
		SendMessage(wnd, TITLE, TRUE, 0);
	   SendMessage(wnd, SETFOCUS, TRUE, 0);
	}
	
	return;
}

static short DosInfoDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	short rtn;
	
	switch (msg)
	{
		case CREATE_WINDOW:
		{
			// If drive letters have already been saved, display them in the
			// dlg box
			rtn = DefaultWndProc(wnd, msg, p1, p2);

			if ( cDosDrivePath || cDosDrivePath[0] )
			{
				PutItemText(wnd, ID_DOSDRIVEPATH, cDosDrivePath);
			}

			return rtn;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_OK:
				{
					char status[55];
					// Need to save drive letters
					GetItemText(wnd, ID_DOSDRIVEPATH, cDosDrivePath, 80);
					
					// If not both drives pop up dlg box saying enter both
					if ( (!cDosDrivePath || !cDosDrivePath[0]) )
					{
						sprintf(status, "You MUST enter the dos path!");
						SendMessage(GetParent(wnd), ADDSTATUS, (PARAM) status, 0);

						return FALSE;
					}
					else
					{
						// If both, close dlg box
						// Let the app close the box for use below
					}
				}
				break;
				case ID_CANCEL:
				{
					// Do nothing, if we had drive letters keep them or keep noghing.
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void RunRawrite( char *fileName )
{
	int imgNumber = -1;
	int i;
	char *tmpversion = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *cmdName = (char *)malloc(sizeof(char) * 30);
	char *args[2];
	char *imgname;
	char *ermsg = (char *)malloc(sizeof(char)*80);
	char *drive;
	int iReturn;
	WINDOW mwnd;
	FILE *fpIn;
	int cdLen;
	char *buf = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *cDelim = ";";


	sprintf(cmdName, "%c:\\dos\\rawrite.exe", cCDRomLetter[0]);

	// Need to find a kernel number that matches them all
	i = 0;

	/* Reread the file to determine img number */
	// Grab a line out of the file
	if ( !buf )
	{
		ErrorMessage("Malloc of buf failed!");
		return;
	}

	mwnd = MomentaryMessage("Determining Image Number...\n     Please wait...");

	fpIn = fopen(fileName, "r");
	if ( !fpIn)
	{
      strcpy(ermsg, "Can not open\n");
      strcat(ermsg, fileName);
      ErrorMessage(ermsg);
      free(ermsg);
		free(buf);
	
		return;
	}
		
	for( ; ; )
	{
		/* Read a line out of the file */
		fgets( buf, MAXLINELEN, fpIn );
			
		/* Check the first character.  If it's a # skip it */
		if ( buf[0] != '#' )
		{
			/* begin parsing and generating the 3 lists. */
			char *cScsiDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cEthDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cCDDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cNumber = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *scsiElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
			char *ethElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
			char *cdElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
	
			/* check for malloc errors! */
			if (!cScsiDriver || !cEthDriver || !cCDDriver || !cNumber
					|| !scsiElem || !ethElem || !cdElem )
			{
		      ErrorMessage("Malloc of cScsiDriver, cEthDriver, cCDDriver or cNumber failed!");
					
				if (cScsiDriver) free(cScsiDriver);
				if (cEthDriver) free(cEthDriver);
				if (cCDDriver) free(cCDDriver);
				if (cNumber) free(cNumber);
				if (scsiElem) free(scsiElem);
				if (ethElem) free(ethElem);
				if (cdElem) free(cdElem);
					
				free(buf);
				free(tmpversion);
				free(cmdName);
				free(ermsg);
					
				return;
			}
	
			/* Parse the string breaking at delim */
			cNumber = strtok(buf,cDelim);
			cScsiDriver = strtok(NULL, cDelim);
			cEthDriver = strtok(NULL,cDelim);
			cCDDriver = strtok(NULL,cDelim);
	
			/* Need to strip \r\n from cCDDriver */
			cdLen = strlen(cCDDriver);
			cCDDriver[cdLen-1] = '\0';
				
         strcpy(scsiElem, cScsiDriver);
         strcpy(ethElem, cEthDriver);
         strcpy(cdElem, cCDDriver);
				
			if ( !strcmp(scsiElem, scsiInclude) && !strcmp(ethElem, ethInclude)
					&& !strcmp(cdElem, cdromInclude) )
			{
				imgNumber = i;

				free(cScsiDriver);
				free(cEthDriver);
				free(cCDDriver);
				free(cNumber);
				free(scsiElem);
				free(ethElem);
				free(cdElem);

				break;
			}
				
			// clean up
			free(cScsiDriver);
			free(cEthDriver);
			free(cCDDriver);
			free(cNumber);

			free(scsiElem);
			free(ethElem);
			free(cdElem);
		}

		i++;		
		/* when we get to eof quit. */
		if (feof(fpIn)) break;
	}

	fclose(fpIn);

	SendMessage(mwnd, CLOSE_WINDOW, 0, 0);

	/* done rereading file */
	if ( imgNumber == -1 )
	{
		ErrorMessage("Couldn't determine image number.");
		return;
	}
	
	/*
		Got our number now spawn rawrite
		MS-DOS prompt> RAWRITE [-f <file>] [-d <drive>] [-n(owait)] [-h(elp)]
			where:	-f <file> - name of disk image file
				-d <drive> - diskette drive to use, must be A or B
				-n - don't prompt for user to insert diskette
				-h - print usage information to stdout
	*/
	
	iReturn = YesNoBox("Please insert the 'Caldera Boot' disk\nPress Yes to create boot disk image");
	
	if (iReturn)
	{
		mwnd = MomentaryMessage("Writing disk...\n     Please wait...");
	
		// Ok make args[0] be the image number

		// Read the version out of version file
		sprintf(tmpversion, "%c:\\images\\version", cCDRomLetter[0]);
		fpIn = fopen(tmpversion, "r");
		fgets( buf, MAXLINELEN, fpIn );
		fclose(fpIn);
		buf[4] = '\0';

		imgname = (char *)malloc(sizeof(char)*MAXLINELEN);
		if (imgNumber < 10)
			sprintf(imgname, "-f %c:\\images\\%s\\boot%d%d%d%d.img", cCDRomLetter[0],buf,0,0,0,imgNumber);
		else if (imgNumber < 100)
			sprintf(imgname, "-f %c:\\images\\%s\\boot%d%d%d.img", cCDRomLetter[0],buf,0,0,imgNumber);

		args[0] = imgname;
		
		// Make args[1] be the floppy drive chosen
		drive = (char *)malloc(sizeof(char)*MAXLINELEN);
		sprintf(drive, "-d %c:", cFloppyLetter[0]);
		args[1] = drive;

		// This is cool 
		args[2] = "-n";
	
		if (!_spawnl(_P_WAIT, cmdName, cmdName, args[0], args[1], args[2], NULL))
		{
			// Terminated normally
			MessageBox("Caldera Network Desktop Utility", "Disk Created Successfully");
		}
		else
		{
			// Terminated abnormally
			ErrorMessage("Disk NOT created successfully");
		}

		SendMessage(mwnd, CLOSE_WINDOW, 0, 0);
      
      free(imgname);
      free(drive);
		free(buf);
	}
	else
	{
		MessageBox("Make Disks", "Not creating boot disk image");
	}


	iReturn = YesNoBox("Please insert the 'Caldera Root' disk \nPress Yes to create the root disk image");
		
	if (iReturn)
	{
		mwnd = MomentaryMessage("Writing disk...\n      Please wait...");
			
		// Ok make args[0] be the image number
		imgname = (char *)malloc(sizeof(char)*MAXLINELEN);
		sprintf(imgname, "-f %c:\\images\\rootdisk.img", cCDRomLetter[0]);
		args[0] = imgname;
			
		// Make args[1] be the floppy drive chosen
		drive = (char *)malloc(sizeof(char)*MAXLINELEN);
		sprintf(drive, "-d %c:", cFloppyLetter[0]);
		args[1] = drive;
	
		// This is cool 
		args[2] = "-n";

		if (!_spawnl(_P_WAIT, cmdName, cmdName, args[0], args[1], args[2], NULL))
		{
			// Terminated normally
			MessageBox("Caldera Network Desktop Utility", "Disk Created Successfully");
		}
		else
		{
			// Terminated abnormally
			ErrorMessage("Disk NOT created successfully");
		}

		SendMessage(mwnd, CLOSE_WINDOW, 0, 0);
		free(imgname);
		free(drive);
	}
	else
	{
		MessageBox("Make Disks", "Not creating root disk image");
	}

	iReturn = YesNoBox("Please insert the 'Caldera Recovery' disk \nPress Yes to create rescue disk image");
		
	if (iReturn)
	{
		mwnd = MomentaryMessage("Writing disk...\n      Please wait...");
			
		// Ok make args[0] be the image number
		imgname = (char *)malloc(sizeof(char)*MAXLINELEN);
		sprintf(imgname, "-f %c:\\images\\rescue.img", cCDRomLetter[0]);
		args[0] = imgname;
			
		// Make args[1] be the floppy drive chosen
		drive = (char *)malloc(sizeof(char)*MAXLINELEN);
		sprintf(drive, "-d %c:", cFloppyLetter[0]);
		args[1] = drive;
	
		// This is cool 
		args[2] = "-n";

		if (!_spawnl(_P_WAIT, cmdName, cmdName, args[0], args[1], args[2], NULL))
		{
			// Terminated normally
			MessageBox("Caldera Network Desktop Utility", "Disk Created Successfully");
		}
		else
		{
			// Terminated abnormally
			ErrorMessage("Disk NOT created successfully");
		}

		SendMessage(mwnd, CLOSE_WINDOW, 0, 0);
		free(imgname);
		free(drive);
	}
	else
	{
		MessageBox("Make Disks", "Not creating rescue disk image");
	}

	free(ermsg);
	free(cmdName);
	free(tmpversion);
	free(buf);
	return;
}

static void SelectScsi(WINDOW wnd)
{
	int iReturn;
	
	// Popup the Select Scsi Dlg box
	iReturn = DialogBox(wnd, &SelectScsiDlg, TRUE, SelectScsiDlgProc);

	return;
}

static short SelectScsiDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	WINDOW cwnd = ControlWindow(&SelectScsiDlg, ID_SCSIDEVICE);
	char *tmp;
	int i;

	switch (msg)
	{
		case INITIATE_DIALOG:
		{
         AddAttribute(cwnd, VSCROLLBAR);

			if ( scsiInclude || scsiInclude[0] )
				PutItemText(wnd, ID_SELECTSCSI, scsiInclude);

			tmp = (char *)first(llScsi);
			while (tmp)
			{
				SendMessage(cwnd, ADDTEXT, (PARAM) (tmp), 0);
	
		      tmp = (char *)next(llScsi);
			}

         SendMessage(cwnd, SHOW_WINDOW, 0, 0);
			break;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_SCSIDEVICE:
					switch ((short) p2)
					{
						case ENTERFOCUS:
						case LB_SELECTION:
							GetDlgListText(wnd, scsiInclude, ID_SCSIDEVICE);
							PutItemText( wnd, ID_SELECTSCSI, scsiInclude);
						break;
						case LB_CHOOSE:
							GetDlgListText(wnd, scsiInclude, ID_SCSIDEVICE);
							SendMessage(wnd, COMMAND, ID_OK, 0);
						break;
					}
				break;
				case ID_OK:
				{
					if (p2 != 0) break;
					GetItemText(wnd, ID_SCSIDEVICE, scsiInclude, MAXFIELDLEN);

					// Ok walk through the lists and make a number
					i = 0;
					tmp = (char *)first(llScsi);
					while (tmp)
					{
						if (!strcmp(tmp, scsiInclude) )
						{
							// Found a match save i
							scsiNum = i;
							break;
						}
						tmp = (char *)next(llScsi);
						i++;
					}
					
					// Must choose a driver or None
					if ( scsiNum == -1 )
					{
						char status[40];
						sprintf(status, "You MUST enter a Scsi Device or None!");
						SendMessage(GetParent(wnd), ADDSTATUS, (PARAM) status, 0);

						return FALSE;
					}
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void SelectEthernet(WINDOW wnd)
{
	int iReturn;
	
	// Popup the select ethernet dlg box
	iReturn = DialogBox(wnd, &SelectEthernetDlg, TRUE, SelectEthernetDlgProc);

	return;
}

static short SelectEthernetDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	WINDOW cwnd = ControlWindow(&SelectEthernetDlg, ID_ETHDEVICE);
	char *tmp;
	int i;

	switch (msg)
	{
		case INITIATE_DIALOG:
		{
         AddAttribute(cwnd, VSCROLLBAR);

			if ( ethInclude || ethInclude[0] )
				PutItemText(wnd, ID_SELECTETH, ethInclude);

			tmp = (char *)first(llEth);
			while (tmp)
			{
				SendMessage(cwnd, ADDTEXT, (PARAM) (tmp), 0);
	
		      tmp = (char *)next(llEth);
			}

         SendMessage(cwnd, SHOW_WINDOW, 0, 0);
			break;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_ETHDEVICE:
					switch ((short) p2)
					{
						case ENTERFOCUS:
						case LB_SELECTION:
							GetDlgListText(wnd, ethInclude, ID_ETHDEVICE);
							PutItemText( wnd, ID_SELECTETH, ethInclude);
						break;
						case LB_CHOOSE:
							GetDlgListText(wnd, ethInclude, ID_ETHDEVICE);
							SendMessage(wnd, COMMAND, ID_OK, 0);
						break;
					}
				break;
				case ID_OK:
				{
					if (p2 != 0) break;
					GetItemText(wnd, ID_ETHDEVICE, ethInclude, MAXFIELDLEN);

					// Ok walk through the lists and make a number
					i = 0;
					tmp = (char *)first(llEth);
					while (tmp)
					{
						if (!strcmp(tmp, ethInclude) )
						{
							// Found a match save i
							ethNum = i;
							break;
						}
						tmp = (char *)next(llEth);
						i++;
					}
					
					// Must choose a driver or None
					if ( ethNum == -1 )
					{
						char status[40];
						sprintf(status, "You MUST enter an Ethernet Device or None!");
						SendMessage(GetParent(wnd), ADDSTATUS, (PARAM) status, 0);

						return FALSE;
					}
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static void SelectCDRom(WINDOW wnd)
{
	int iReturn;
	
	// Popup the Select CDRom dlg box
	iReturn = DialogBox(wnd, &SelectCDRomDlg, TRUE, SelectCDRomDlgProc);

	return;
}

static short SelectCDRomDlgProc(WINDOW wnd, MESSAGE msg, PARAM p1, PARAM p2)
{
	WINDOW cwnd = ControlWindow(&SelectCDRomDlg, ID_CDROMDEVICE);
	char *tmp;
	int i;

	switch (msg)
	{
		case INITIATE_DIALOG:
		{
         AddAttribute(cwnd, VSCROLLBAR);

			if ( cdromInclude || cdromInclude[0] )
				PutItemText(wnd, ID_SELECTCDROM, cdromInclude);

			tmp = (char *)first(llCDRom);
			while (tmp)
			{
				SendMessage(cwnd, ADDTEXT, (PARAM) (tmp), 0);
	
		      tmp = (char *)next(llCDRom);
			}

         SendMessage(cwnd, SHOW_WINDOW, 0, 0);
			break;
		}
		break;
		case COMMAND:
		{
			switch ( (short)p1 )
			{
				case ID_CDROMDEVICE:
					switch ((short) p2)
					{
						case ENTERFOCUS:
						case LB_SELECTION:
							GetDlgListText(wnd, cdromInclude, ID_CDROMDEVICE);
							PutItemText( wnd, ID_SELECTCDROM, cdromInclude);
						break;
						case LB_CHOOSE:
							GetDlgListText(wnd, cdromInclude, ID_CDROMDEVICE);
							SendMessage(wnd, COMMAND, ID_OK, 0);
						break;
					}
				break;
				case ID_OK:
				{
					if (p2 != 0) break;
					GetItemText(wnd, ID_CDROMDEVICE, cdromInclude, MAXFIELDLEN);

					// Ok walk through the lists and make a number
					i = 0;
					tmp = (char *)first(llCDRom);
					while (tmp)
					{
						if (!strcmp(tmp, cdromInclude) )
						{
							// Found a match save i
							cdNum = i;
							break;
						}
						tmp = (char *)next(llCDRom);
						i++;
					}
					
					// Must choose a driver or None
					if ( cdNum == -1 )
					{
						char status[40];
						sprintf(status, "You MUST enter a CD-Rom Device or None!");
						SendMessage(GetParent(wnd), ADDSTATUS, (PARAM) status, 0);

						return FALSE;
					}
				}
				break;
				case ID_CANCEL:
				{
				}
				break;
			}
		}
		break;
		default:
		{
		}
		break;
	}

	return DefaultWndProc(wnd, msg, p1, p2);
}

static int parseFile( char *fileName )
{
	FILE *fpIn;
	char *cDelim = ";";
	char *buf = (char *)malloc(sizeof(char) * MAXLINELEN);
	int iLen = 0;
	char *ermsg = (char *)malloc(sizeof(char) * MAXLINELEN);
	char *scsiElem;
	char *ethElem;
	char *cdElem;
	int cdLen = 0;
	char *tmp;
	int inthere;

	if ( !buf )
	{
		ErrorMessage("parseFile: Malloc of buf failed!");
		return 0;
	}
	
	fpIn = fopen(fileName, "r");
	if ( !fpIn)
	{
      strcpy(ermsg, "Can not open\n");
      strcat(ermsg, fileName);
      ErrorMessage(ermsg);
		free(buf);
		free(ermsg);

		return 0;
	}

	/* Now parse through the file.
	   Lines beginning with # are comments
	   Other lines are in the form XXXXX; SCSI; ETH; CDROM */
	
	for( ; ; )
	{
		/* Read a line out of the file */
		fgets( buf, MAXLINELEN, fpIn );
		
		/* Get actual length of the string it includes \n */
		iLen = strlen(buf);

		/* Check the first character.  If it's a # skip it */
		if ( buf[0] != '#' )
		{
			/* begin parsing and generating the 3 lists. */
			char *cScsiDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cEthDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cCDDriver = (char *)malloc(sizeof(char) * MAXFIELDLEN);
			char *cNumber = (char *)malloc(sizeof(char) * MAXFIELDLEN);

			/* check for malloc errors! */
			if (!cScsiDriver || !cEthDriver || !cCDDriver || !cNumber )
			{
		      ErrorMessage("Malloc of cScsiDriver, cEthDriver, cCDDriver or cNumber failed!");
				
				if (cScsiDriver) free(cScsiDriver);
				if (cEthDriver) free(cEthDriver);
				if (cCDDriver) free(cCDDriver);
				if (cNumber) free(cNumber);
				free(buf);
				free(ermsg);
				
				return 0;
			}

			/* Parse the string breaking at delim */
			cNumber = strtok(buf,cDelim);
			cScsiDriver = strtok(NULL, cDelim);
			cEthDriver = strtok(NULL,cDelim);
			cCDDriver = strtok(NULL,cDelim);

			/* Need to strip \r\n from cCDDriver */
			cdLen = strlen(cCDDriver);
			cCDDriver[cdLen-1] = '\0';
			
			/* Make a list of strings */
			scsiElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(scsiElem, cScsiDriver);
         
			// should we add?
			inthere = FALSE;
			tmp = first(llScsi);
			while ( tmp )
			{
				if (!strcmp(scsiElem, tmp))
					inthere = TRUE;
				
				tmp = next(llScsi);
			}
			
			if (!inthere)
				add(llScsi, scsiElem);

			ethElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(ethElem, cEthDriver);
         
			// should we add?
			inthere = FALSE;
			tmp = first(llEth);
			while ( tmp )
			{
				if (!strcmp(ethElem, tmp))
					inthere = TRUE;
				
				tmp = next(llEth);
			}
			
			if (!inthere)
				add(llEth, ethElem);

			cdElem = (char *)malloc(sizeof(char)*MAXFIELDLEN);
         strcpy(cdElem, cCDDriver);

			// should we add?
			inthere = FALSE;
			tmp = first(llCDRom);
			while ( tmp )
			{
				if (!strcmp(cdElem, tmp))
					inthere = TRUE;
				
				tmp = next(llCDRom);
			}
			
			if (!inthere)
				add(llCDRom, cdElem);
			
			// clean up
			free(cScsiDriver);
			free(cEthDriver);
			free(cCDDriver);
			free(cNumber);
		}
		
		/* when we get to eof quit. */
		if (feof(fpIn)) break;
	}
	
	free(buf);
	free(ermsg);
	fclose(fpIn);

	return 1;	
}
